@@ -8,8 +8,12 @@
 #  Email: rcabdia@roa.es
 # --------------------------------------------------------------------
 
+import warnings
 import os
+os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"
+warnings.filterwarnings("ignore")
 import sys
+import traceback
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
 from dataclasses import dataclass
 from dateutil import parser
@@ -19,6 +23,8 @@ from surfquakecore.arrayanalysis.beamrun import TraceBeamResult
 from surfquakecore.data_processing.analysis_events import AnalysisEvents
 from surfquakecore.data_processing.processing_methods import print_surfquake_trace_headers
 from surfquakecore.earthquake_location.run_nll import NllManager, Nllcatalog
+from surfquakecore.first_polarity.first_polarity import FirstPolarity
+from surfquakecore.first_polarity.get_pol import RunPolarity
 from surfquakecore.magnitudes.run_magnitudes import Automag
 from surfquakecore.magnitudes.source_tools import ReadSource
 from surfquakecore.moment_tensor.mti_parse import WriteMTI, BuildMTIConfigs
@@ -91,7 +97,17 @@ def _create_actions():
             name="info", run=_info, description=f"Type {__entry_point_name} -h for help.\n"),
 
         "explore": _CliActions(
-            name="explore", run=_explore, description=f"Type {__entry_point_name} -h for help.\n")
+            name="explore", run=_explore, description=f"Type {__entry_point_name} -h for help.\n"),
+
+        "polarity": _CliActions(
+            name="polarity", run=_polarity, description=f"Type {__entry_point_name} -h for help.\n"),
+
+        "focmec": _CliActions(
+            name="focmec", run=_focmec, description=f"Type {__entry_point_name} -h for help.\n"),
+
+        "plotmec": _CliActions(
+            name="plotmec", run=_plotmec, description=f"Type {__entry_point_name} -h for help.\n")
+
     }
 
     return _actions
@@ -175,6 +191,7 @@ def _pick():
         description="Use PhaseNet deep learning model to pick P- and S-wave arrivals.",
         formatter_class=RawDescriptionHelpFormatter,
         epilog="""
+    
     Overview:
         This command applies the PhaseNet deep neural network to estimate the arrival times
         of P- and S-waves in local and regional seismic events.
@@ -192,8 +209,13 @@ def _pick():
     Key Arguments:
         -f, --project_file        Path to your seismic project file
         -d, --output_dir          Directory to save pick results
-        -p, --p_thresh            Threshold for P-wave probability (0–1) (default: 0.3)
-        -s, --s_thresh            Threshold for S-wave probability (0–1) (default: 0.3)
+        -pt, --p_thresh            [OPTIONAL] Threshold for P-wave probability (0–1) (default: 0.3)
+        -st, --s_thresh            [OPTIONAL] Threshold for S-wave probability (0–1) (default: 0.3)
+        -n, --net                 [OPTIONAL] Network code filter
+        -s, --station             [OPTIONAL] Station code filter
+        -ch, --channel            [OPTIONAL] Channel filter
+        --min_date                [OPTIONAL] Filter Start date (format: YYYY-MM-DD HH:MM:SS), DEFAULT min date of the project
+        --max_date                [OPTIONAL] Filter End date   (format: YYYY-MM-DD HH:MM:SS), DEFAULT max date of the project
         --verbose                 Enable detailed logging
 
     Reference:
@@ -211,16 +233,48 @@ def _pick():
 
     arg_parse.add_argument("-f", help="Path to your project file", type=str, required=True)
     arg_parse.add_argument("-d", help="Path to directory where picks will be saved", type=str, required=True)
-
-    arg_parse.add_argument("-p", "--p_thresh", help="P-wave threshold", type=float, default=0.3)
-    arg_parse.add_argument("-s", "--s_thresh", help="S-wave threshold", type=float, default=0.3)
-
+    arg_parse.add_argument("-n", "--net", help="Network code filter", type=str)
+    arg_parse.add_argument("-s", "--station", help="Station code filter", type=str)
+    arg_parse.add_argument("-ch", "--channel", help="Channel code filter", type=str)
+    arg_parse.add_argument("-pt", "--p_thresh", help="P-wave threshold", type=float, default=0.3)
+    arg_parse.add_argument("-st", "--s_thresh", help="S-wave threshold", type=float, default=0.3)
+    arg_parse.add_argument("--min_date", help="Start time filter: format 'YYYY-MM-DD HH:MM:SS.sss'", type=str)
+    arg_parse.add_argument("--max_date", help="End time filter: format 'YYYY-MM-DD HH:MM:SS.sss'", type=str)
     arg_parse.add_argument("-v", "--verbose", help="Show detailed log output", action="store_true")
 
     parsed_args = arg_parse.parse_args()
     print(parsed_args)
 
     sp_loaded = SurfProject.load_project(path_to_project_file=make_abs(parsed_args.f))
+
+    # --- Apply key filters ---
+    filters = {}
+    if parsed_args.net:
+        filters["net"] = parsed_args.net
+    if parsed_args.station:
+        filters["station"] = parsed_args.station
+    if parsed_args.channel:
+        filters["channel"] = parsed_args.channel
+    if filters:
+        print(f"[INFO] Filtering project by: {filters}")
+        sp_loaded.filter_project_keys(**filters)
+
+        # --- Apply time filters ---
+        min_date, max_date = None, None
+        try:
+            if parsed_args.min_date:
+                # min_date = datetime.strptime(args.min_date, "%Y-%m-%d %H:%M:%S.%f")
+                min_date = parser.parse(parsed_args.min_date)
+            if parsed_args.max_date:
+                # max_date = datetime.strptime(args.max_date, "%Y-%m-%d %H:%M:%S.%f")
+                max_date = parser.parse(parsed_args.max_date)
+            if min_date or max_date:
+                print(f"[INFO] Filtering by time range: {min_date} to {max_date}")
+                sp_loaded.filter_project_time(starttime=min_date, endtime=max_date, verbose=True)
+        except ValueError as ve:
+            print(f"[ERROR] Date format should be: 'YYYY-MM-DD HH:MM:SS.sss'")
+            raise ve
+
     if len(sp_loaded.project) > 0 and isinstance(sp_loaded, SurfProject):
         picker = PhasenetISP(
             sp_loaded.project,
@@ -242,6 +296,172 @@ def _pick():
         print("Empty Project, Nothing to pick!")
 
 
+def _polarity():
+    parser = ArgumentParser(
+        prog="surfquake polarity",
+        description="Determines Polarities from P-wave first motion",
+        formatter_class=RawDescriptionHelpFormatter,
+        epilog="""
+        
+        Overview:
+        Automatic P-wave first motion polarity determination. The inputs are the project and the picking file in 
+        nonlinloc pick format (i.e., might be generated using command pick). 
+        Output is an edited pick file with polarities.
+        
+        Reference:
+        Chakraborty, M., Cartaya, C. Q., Li, W., Faber, J., Rümpker, G., Stoecker, H., & Srivastava, N. (2022). 
+        PolarCAP–A deep learning approach for first motion polarity classification of earthquake waveforms. 
+        Artificial Intelligence in Geosciences, 3, 46-52.
+        
+        Example usage:
+        surfquake polarity -f './nll_picks.txt' -p './project_file.pkl' -o './nll_picks_polarities.txt' -t 0.95 
+        
+
+        """
+    )
+
+    parser.add_argument("-p", "--project_file_path", required=True, help="Path to SurfProject .pkl")
+    parser.add_argument("-f", "--picking_file_path", required=True, help="path to picking file", type=str)
+    parser.add_argument("-o", "--output_file_path", required=True, help="output file", type=str)
+    parser.add_argument("-t", "--thresh", required=False, help="P-wave threshold", type=float, default=0.9)
+
+    args = parser.parse_args()
+
+    picking_file = make_abs(args.picking_file)
+    project_file = make_abs(args.project_file)
+
+    project = SurfProject.load_project(project_file)
+    RunPolarity(project, picking_file, args.output_file, args.thresh).send_polarities()
+
+
+def _focmec():
+    parser = ArgumentParser(
+        prog="surfquake focmec",
+        description="Focal Mechanism from P-Wave first motion polarity",
+        formatter_class=RawDescriptionHelpFormatter,
+        epilog="""
+        
+        Overview:
+        
+        FOCal MEChanism determinations (FOCMEC) software  for determining and 
+        displaying double-couple earthquake focal mechanisms using the polarity and amplitude ratios of P and S waves.
+        
+        Reference:
+        Snoke, J. A. (2003). “FOCMEC: Focal Mechanism Determinations.” 
+        International Handbook of Earthquake and Engineering Seismology, Part B, Academic Press, pp. 1629–1630.
+        https://seiscode.iris.washington.edu/projects/focmec
+        
+        Example usage:
+        > surfquake focmec -d './folder_hyp_path' -a 1.0 -o './output_folder'
+        
+        """
+    )
+
+    parser.add_argument("-d", "--hyp_folder", required=True, help="path to folder containing hyp files",
+                        type=str)
+    parser.add_argument("-a", "--accepted", required=False, help="Number of accepted wrong polarities",
+                        type=float, default=1.0)
+    parser.add_argument("-o", "--output_folder", required=False, help="output folder", type=str)
+
+
+    args = parser.parse_args()
+
+    hyp_folder = make_abs(args.hyp_folder)
+    output_folder = make_abs(args.output_folder)
+    files_list = FirstPolarity.find_hyp_files(hyp_folder)
+    for file in files_list:
+        try:
+            header = FirstPolarity.set_head(file)
+            if file is not None:
+                file_input = FirstPolarity().create_input(file, header)
+
+                if FirstPolarity.check_no_empty(file_input):
+                    FirstPolarity().run_focmec(file_input, 3, output_folder)
+        except Exception as e:
+            print(f"Error processing file {file}: {e}")
+            traceback.print_exc()
+
+def _plotmec():
+    parser = ArgumentParser(
+        prog="surfquake plotmec",
+        description="Focal Mechanism from P-wave polarity ",
+        formatter_class=RawDescriptionHelpFormatter,
+        epilog="""
+        Overview:
+        
+        Plot fault planes, T & P axis, and P-wave polarities.
+        
+        Example usage:
+
+        surfquake plotmec -d './focmec_folder_path' -o './output_folder'
+        surfquake plotmec -f './focmec_file_path' -o './output_folder'
+            
+        if output is not provided the beachball of the focal mechanism will be shown on screen, 
+        but if user provide output folder, the beach ball plot will be saved in the folder
+        
+            """
+    )
+    parser.add_argument("-f", "--focmec_file", required=False,
+                        help="file with focmec.lst solution", type=str)
+
+    parser.add_argument("-d", "--focmec_folder_path", required=False, help="path to folder with all *.lst "
+                                                                           "focmec solutions", type=str)
+    parser.add_argument("-o", "--output_folder", required=False, help="output folder", type=str)
+    parser.add_argument("-m", "--format", required=False, help="format output plot (defaults pdf)",
+                        type=str, default="pdf")
+    args = parser.parse_args()
+
+    firstpolarity_manager = FirstPolarity()
+
+    if args.focmec_file:
+        focmec_file = make_abs(args.focmec_file)
+        focmec_files = [focmec_file]
+    else:
+        focmec_files = firstpolarity_manager.find_files(make_abs(args.focmec_folder_path))
+
+    format = args.format
+
+    for file in focmec_files:
+
+        # Station, Az, Dip, Motion = firstpolarity_manager.get_dataframe(location_file)
+        Station, Az, Dip, Motion = FirstPolarity.extract_station_data(file)
+        cat, focal_mechanism = firstpolarity_manager.extract_focmec_info(file)
+        # TODO MIGHT BE FOR PLOTTING ALL POSSIBLE FAUL PLANES
+        # focmec_full_Data = parse_focmec_file(focmec_file)
+        file_output_name = FirstPolarity.extract_name(file)
+
+        if args.output_folder:
+            output_folder = make_abs(args.output_folder)
+            name_str = os.path.basename(file)[:-3] + format
+            output_folder_file = os.path.join(output_folder, name_str)
+        else:
+            output_folder_file = None
+
+        Plane_A = focal_mechanism.nodal_planes.nodal_plane_1
+        strike_A = Plane_A.strike
+        dip_A = Plane_A.dip
+        rake_A = Plane_A.rake
+        extra_info = firstpolarity_manager.parse_solution_block(focal_mechanism.comments[0]["text"])
+        P_Trend = extra_info['P,T']['Trend']
+        P_Plunge = extra_info['P,T']['Plunge']
+        T_Trend = extra_info['P,N']['Trend']
+        T_Plunge = extra_info['P,N']['Plunge']
+
+        misfit_first_polarity = focal_mechanism.misfit
+        azimuthal_gap = focal_mechanism.azimuthal_gap
+        number_of_polarities = focal_mechanism.station_polarity_count
+        #
+        first_polarity_results = {"First_Polarity": ["Strike", "Dip", "Rake", "misfit_first_polarity", "azimuthal_gap",
+                                                     "number_of_polarities", "P_axis_Trend", "P_axis_Plunge",
+                                                     "T_axis_Trend", "T_axis_Plunge"],
+                                  "results": [strike_A, dip_A, rake_A, misfit_first_polarity, azimuthal_gap,
+                                              number_of_polarities, P_Trend, P_Plunge, T_Trend, T_Plunge]}
+
+        FirstPolarity.print_first_polarity_info(file_output_name, first_polarity_results)
+        FirstPolarity.drawFocMec(strike_A, dip_A, rake_A, Station, Az, Dip, Motion, P_Trend, P_Plunge,
+                                              T_Trend, T_Plunge, output_folder_file)
+
+
 def _associate():
 
     arg_parse = ArgumentParser(
@@ -841,6 +1061,7 @@ def _processing():
             - Apply processing steps (filtering, normalization, etc.)
             - Optionally visualize the waveforms (interactive mode)
             - Apply a user-defined post-processing script before or after plotting
+            - Warning: The process will automatically merge traces
 
     Modes:
         Default  : Interactive mode (plotting + user prompts)
@@ -884,6 +1105,7 @@ def _processing():
         --plot_config              [OPTIONAL] Optional plot configuration file (YAML)
         --post_script              [OPTIONAL] Python script to apply per event stream
         --post_script_stage        [OPTIONAL] When to apply the post-script: before | after (default: before)
+    
     """
     )
 
@@ -1202,7 +1424,8 @@ def _quickproc():
         description="Quick waveform processing. Designed for rapid, raw file-based workflows.",
         formatter_class=RawDescriptionHelpFormatter,
         epilog="""
-Overview:
+    
+    Overview:
         Process seismic traces.
         You can:
             - Apply processing steps (filtering, normalization, etc.)
@@ -1412,19 +1635,29 @@ def _info():
         description="Prints waveform information ",
         formatter_class=RawDescriptionHelpFormatter,
         epilog="""
+        
+        Overview:
+        
+        Extracts and prints on screen the traces header information
+        
+        Key Arguments:
+            -w, --wave_files   [REQUIRED] Glob pattern or path to waveform files
+            -c, --columns      [OPTIONAL] Number of columns showing traces info (default = 5)
+
         Example usage:
 
-  surfquake info -w './data/*.mseed' -c 3
-  surfquake info -w 'trace1.mseed, trace2.mseed' --columns 5
+    surfquake info -w './data/*.mseed' -c 3
+    surfquake info -w 'trace1.mseed,trace2.mseed' --columns 5
 
-Supports standard and SurfQuake-extended headers such as picks, references, and geodetic attributes.
+    Supports standard and SurfQuake-extended headers such as picks, references, and geodetic attributes.
 
     """
     )
 
-    parser.add_argument("-w", "--wave_files", help="path to waveform files (e.g. './data/*Z')", type=str)
-    parser.add_argument("-c", "--columns",help="Number of traces (columns) to show per file (default = 5).",
+    parser.add_argument("-w", "--wave_files", required=True, help="path to waveform files (e.g. './data/*Z')", type=str)
+    parser.add_argument("-c", "--columns", help="Number of traces (columns) to show per file (default = 5).",
                         type=int, default=1)
+
     args = parser.parse_args()
 
     if "," in args.wave_files or " " in args.wave_files:
@@ -1441,31 +1674,92 @@ Supports standard and SurfQuake-extended headers such as picks, references, and
 
 
 def _explore():
-    parser = ArgumentParser(
-        prog="surfquake info",
+
+    arg_parse = ArgumentParser(
+        prog="surfquake explore",
         description="Explore data availability ",
         formatter_class=RawDescriptionHelpFormatter,
         epilog="""
-        Example usage:
+    
+    Overview:
+    
+    Plot the data availability of your seismogram files
+    
+    Key Arguments:
+            -w, --wave_files          [OPTIONAL] Glob pattern or path to waveform files
+            -p, --project_path_file   [OPTIONAL] Project path file
+            -n, --net                 [OPTIONAL] Network code filter (if -p selected)
+            -s, --station             [OPTIONAL] Station code filter (if -p selected)
+            -ch, --channel            [OPTIONAL] Channel filter (if -p selected)
+            --min_date                [OPTIONAL] Filter Start date (format: YYYY-MM-DD HH:MM:SS), DEFAULT min date of the project (if -p selected)
+            --max_date                [OPTIONAL] Filter End date   (format: YYYY-MM-DD HH:MM:SS), DEFAULT max date of the project (if -p selected)
+            
+    Example usage:
 
-  surfquake info -w './data/*.mseed'
-  
+    surfquake explore -w './data/*.mseed'
+    surfquake explore -p './project.pkl'
     """
     )
 
-    parser.add_argument("-w", "--wave_files", help="path to waveform files (e.g. './data/*Z')", type=str)
-    args = parser.parse_args()
+    arg_parse.add_argument("-w", "--wave_files", help="path to waveform files (e.g. './data/*Z')", type=str)
+    arg_parse.add_argument("-p", "--project_file_path", help="scan files from the project", type=str)
+    arg_parse.add_argument("-n", "--net", help="Network code filter", type=str)
+    arg_parse.add_argument("-s", "--station", help="Station code filter", type=str)
+    arg_parse.add_argument("-ch", "--channel", help="Channel code filter", type=str)
+    arg_parse.add_argument("--min_date", help="Start time filter: format 'YYYY-MM-DD HH:MM:SS.sss'", type=str)
+    arg_parse.add_argument("--max_date", help="End time filter: format 'YYYY-MM-DD HH:MM:SS.sss'", type=str)
 
-    if "," in args.wave_files or " " in args.wave_files:
-        # Explicit list of files
-        wave_paths = [make_abs(f.strip()) for f in args.wave_files.split(",") if f.strip()]
+    args = arg_parse.parse_args()
+
+    if args.project_file_path:
+        project_file_path = make_abs(args.project_file_path)
+        sp = SurfProject.load_project(project_file_path)
+
+        # --- Apply key filters ---
+        filters = {}
+        if args.net:
+            filters["net"] = args.net
+        if args.station:
+            filters["station"] = args.station
+        if args.channel:
+            filters["channel"] = args.channel
+        if filters:
+            print(f"[INFO] Filtering project by: {filters}")
+            sp.filter_project_keys(**filters)
+
+            # --- Apply time filters ---
+            min_date, max_date = None, None
+            try:
+                if args.min_date:
+                    # min_date = datetime.strptime(args.min_date, "%Y-%m-%d %H:%M:%S.%f")
+                    min_date = parser.parse(args.min_date)
+                if args.max_date:
+                    # max_date = datetime.strptime(args.max_date, "%Y-%m-%d %H:%M:%S.%f")
+                    max_date = parser.parse(args.max_date)
+                if min_date or max_date:
+                    print(f"[INFO] Filtering by time range: {min_date} to {max_date}")
+                    sp.filter_project_time(starttime=min_date, endtime=max_date, verbose=True)
+            except ValueError as ve:
+                print(f"[ERROR] Date format should be: 'YYYY-MM-DD HH:MM:SS.sss'")
+                raise ve
+
+        data_files = []
+        for item in sp.project.items():
+            list_channel = item[1]
+            for file_path in list_channel:
+                data_files.append(file_path[0])
 
     else:
-        # Wildcard path
-        wave_paths = make_abs(args.wave_files)
-     # Build project on-the-fly using wildcard path
+        if "," in args.wave_files or " " in args.wave_files:
+            # Explicit list of files
+            wave_paths = [make_abs(f.strip()) for f in args.wave_files.split(",") if f.strip()]
 
-    data_files = SurfProject.collect_files(root_path=wave_paths)
+        else:
+            # Wildcard path
+            wave_paths = make_abs(args.wave_files)
+         # Build project on-the-fly using wildcard path
+
+        data_files = SurfProject.collect_files(root_path=wave_paths)
     PlotExplore.data_availability_new(data_files)
 
 
