@@ -498,6 +498,8 @@ class SurfProject:
         if isinstance(starttime, str):
             start = datetime.strptime(starttime, date_format)
             start = UTCDateTime(start)
+        elif isinstance(starttime, datetime):
+            start = UTCDateTime(starttime)
         elif isinstance(starttime, UTCDateTime):
             start = starttime
         else:
@@ -506,6 +508,8 @@ class SurfProject:
         if isinstance(endtime, str):
             end = datetime.strptime(endtime, date_format)
             end = UTCDateTime(end)
+        elif isinstance(endtime, datetime):
+            end = UTCDateTime(endtime)
         elif isinstance(endtime, UTCDateTime):
             end = endtime
         else:
@@ -574,9 +578,7 @@ class SurfProject:
 
         else:
 
-            # for file in data_files:
-
-            for file in self.data_files:
+            for file in data_files:
 
                 pos_file = file[0]
                 st0 = file[1]
