@@ -9,7 +9,8 @@ import os
 import readline
 import atexit
 from typing import Optional
-from surfquakecore.data_processing.processing_methods import filter_trace
+from surfquakecore.data_processing.processing_methods import filter_trace, print_surfquake_trace_headers
+
 
 class PlotCommandPrompt:
     def __init__(self, plot_proj):
@@ -34,12 +35,15 @@ class PlotCommandPrompt:
             "pm": self._cmd_pm,
             "p": self._cmd_pick,
             "beam": self._cmd_beam,
+            "smap": self._cmd_smap,
             "xcorr": self._cmd_xcorr,
             "plot_type": self._cmd_type,
             "cut": self._cmd_cut,
             "concat": self._cmd_concat,
+            "load_picks": self._cmd_load_picks,
             "shift": self._cmd_shift,
             "write": self._cmd_write,
+            "info": self.plot_command_prompt,
             "help": self._cmd_help,
             "exit": self._cmd_exit
         }
@@ -47,47 +51,46 @@ class PlotCommandPrompt:
     def make_abs(self, path: Optional[str]) -> Optional[str]:
         return os.path.abspath(path) if path else None
 
-
     def _cmd_exit(self, args):
 
-         import warnings
-         warnings.filterwarnings("ignore", message="resource_tracker: There appear to be")
+        import warnings
+        warnings.filterwarnings("ignore", message="resource_tracker: There appear to be")
 
-    #     """
-    #     Exit the plot and close the interactive session.
-    #     Usage: exit
-    #     """
+        #     """
+        #     Exit the plot and close the interactive session.
+        #     Usage: exit
+        #     """
 
-         import matplotlib.pyplot as plt
-         import gc
-         import multiprocessing as mp
-         import warnings
+        import matplotlib.pyplot as plt
+        import gc
+        import multiprocessing as mp
+        import warnings
 
-         print("[INFO] Exiting interactive plotting session.")
-         self.prompt_active = False
-         self._exit_code = "exit"
-    #
-         try:
-             if self.plot_proj and hasattr(self.plot_proj, "fig"):
-                 plt.close(self.plot_proj.fig)
-                 self.plot_proj.fig = None
-         except Exception:
-             pass
+        print("[INFO] Exiting interactive plotting session.")
+        self.prompt_active = False
+        self._exit_code = "exit"
+        #
+        try:
+            if self.plot_proj and hasattr(self.plot_proj, "fig"):
+                plt.close(self.plot_proj.fig)
+                self.plot_proj.fig = None
+        except Exception:
+            pass
 
-         # Explicitly call garbage collector to release mpl backends/semaphores
-         gc.collect()
+        # Explicitly call garbage collector to release mpl backends/semaphores
+        gc.collect()
 
-         # On macOS, also consider forcibly terminating child processes if you're using any (e.g., in FK)
-         try:
-             mp.active_children()
-             mp.get_context().get_logger().setLevel("ERROR")  # suppress log noise
-         except Exception:
-             pass
+        # On macOS, also consider forcibly terminating child processes if you're using any (e.g., in FK)
+        try:
+            mp.active_children()
+            mp.get_context().get_logger().setLevel("ERROR")  # suppress log noise
+        except Exception:
+            pass
 
-         # Suppress final cleanup warning from multiprocessing
-         warnings.filterwarnings("ignore", message="resource_tracker: There appear to be")
+        # Suppress final cleanup warning from multiprocessing
+        warnings.filterwarnings("ignore", message="resource_tracker: There appear to be")
 
-         print("[INFO] Cleanup complete.")
+        print("[INFO] Cleanup complete.")
 
     def run(self) -> str:
         """
@@ -100,7 +103,7 @@ class PlotCommandPrompt:
             - 'next': User typed 'n' to continue
             - 'pick': User wants to return to picking mode
         """
-        
+
         if self.prompt_active:
             return "next"  # Prevent re-entry
 
@@ -138,6 +141,61 @@ class PlotCommandPrompt:
         self.plot_proj.plot(page=self.plot_proj.current_page)  # replot
         # Do not set prompt_active = False; stay in prompt!
 
+    def _cmd_load_picks(self, args):
+
+        """
+        Load picks from an ISP/CSV-like table (with headers) and draw them.
+        Usage:
+            loadisp --file <path> [--delim <regex_or_char>]
+        """
+        path = None
+        delim = r"\s+"
+
+        it = iter(args[1:])
+        for arg in it:
+            if arg == "--file":
+                try:
+                    path = os.path.abspath(next(it))
+                except StopIteration:
+                    print("[ERROR] Missing value after --file")
+                    return
+            elif arg == "--delim":
+                try:
+                    delim = next(it)
+                except StopIteration:
+                    print("[ERROR] Missing value after --delim")
+                    return
+
+        if not path:
+            print("[ERROR] Use: loadisp --file <path> [--delim <regex_or_char>]")
+            return
+        if not os.path.exists(path):
+            print(f"[ERROR] File not found: {path}")
+            return
+
+        try:
+            self.plot_proj.import_nlloc_obs(path, delimiter=delim)
+            if getattr(self.plot_proj, "fig", None):
+                self.plot_proj._redraw_picks()
+                self.plot_proj._update_info_box()
+                self.plot_proj.fig.canvas.draw_idle()
+
+                print("[INFO] Returning to picking mode...")
+                self.prompt_active = False
+                self._exit_code = "p"
+        except Exception as e:
+            print(f"[ERROR] Failed to load ISP picks: {e}")
+
+    def plot_command_prompt(self, args):
+
+        traces = getattr(self.plot_proj, "trace_list", [])
+
+        try:
+            print_surfquake_trace_headers(traces, max_columns=3)
+        except Exception as e:
+            print(f"Error displaying info: {e}")
+
+
     def _cmd_filter(self, args):
         """
         Apply a filter to currently displayed traces.
@@ -299,6 +357,56 @@ class PlotCommandPrompt:
         except ValueError:
             print("Error: index, param, fmin, fmax, and clip must be numeric where applicable.")
 
+    def _cmd_smap(self, args):
+        """
+        Run slowness map analysis and show output.
+
+        Usage: smap [--method fk] [--fmin 0.8][--fmax 2.2] [--smax 0.3] [--grid 0.05] [--nsignals 1]
+        """
+        # Default parameters
+        params = {
+            "method": "FK",
+            "fmin": 0.8,
+            "fmax": 2.2,
+            "smax": 0.3,
+            "slow_grid": 0.01,
+            "nsignals": 1}
+
+        # Allowed keys and their types
+        valid_keys = {
+            "method": str,
+            "fmin": float,
+            "fmax": float,
+            "smax": float,
+            "grid": float,
+            "nsignals": int}  # maps to slow_grid
+
+        # Parse arguments
+        it = iter(args[1:])  # Skip 'beam'
+        for arg in it:
+            if arg.startswith("--"):
+                key = arg[2:]
+                if key not in valid_keys:
+                    print(f"[WARNING] Unknown option '--{key}' ignored.")
+                    continue
+                try:
+                    val_str = next(it)
+                    val = valid_keys[key](val_str)
+                    if key == "grid":
+                        params["slow_grid"] = val
+                    else:
+                        params[key] = val
+                except (StopIteration, ValueError):
+                    print(f"[ERROR] Invalid value for --{key}")
+                    return
+
+        print(f"[INFO] Running beamforming method '{params['method']}' with parameters: {params}")
+
+        try:
+            self.plot_proj._slowness_map(**params)
+        except Exception as e:
+            print(f"[ERROR] Beamforming run failed: {e}")
+
     def _cmd_beam(self, args):
         """
         Run beamforming analysis and show output.
@@ -388,7 +496,8 @@ class PlotCommandPrompt:
 
         new_mode = args[1].lower()
         if new_mode not in self.plot_proj.available_modes:
-            print(f"[ERROR] Unknown mode '{new_mode}'. Available plot types: {', '.join(self.plot_proj.available_types)}")
+            print(
+                f"[ERROR] Unknown mode '{new_mode}'. Available plot types: {', '.join(self.plot_proj.available_types)}")
             return
 
         self.plot_proj.plot_config["plot_type"] = new_mode
@@ -585,7 +694,8 @@ class PlotCommandPrompt:
                 except Exception as e:
                     print(f"[ERROR] Could not cut {tr.id}: {e}")
 
-        elif len(args)==1 and isinstance(self.plot_proj.utc_start, UTCDateTime) and isinstance(self.plot_proj.utc_end, UTCDateTime):
+        elif len(args) == 1 and isinstance(self.plot_proj.utc_start, UTCDateTime) and isinstance(self.plot_proj.utc_end,
+                                                                                                 UTCDateTime):
             for tr in self.plot_proj.trace_list:
                 try:
                     tr_cut = tr.copy().trim(starttime=self.plot_proj.utc_start, endtime=self.plot_proj.utc_end,
@@ -890,22 +1000,32 @@ class PlotCommandPrompt:
             >> cut --reference 5 20
             >> cut --start "2023-01-01 12:00:00" --end "2023-01-01 12:01:00"
     """,
-            "beam": """
-            beam [--fmin <Hz>] [--fmax <Hz>] [--smax <s/km>] [--grid <step>] [--win <s>] [--overlap <ratio>] [--method fk]
-                Run FK (or other) beamforming method on current traces.
 
-                Options:
-                    --method    : Beamforming type ('FK', 'MTP.COHERENCE', 'CAPON' or 'MUSIC)
+            "beam": """
+            
+            Run FK beamforming method in sliding windows on current traces.
+            
+            beam [--fmin <Hz>] [--fmax <Hz>] [--smax <s/km>] [--grid <step>] [--win <s>] [--overlap <ratio>]
+                
+                    Options:
                     --fmin      : Min frequency (Hz)
                     --fmax      : Max frequency (Hz)
                     --smax      : Max slowness (s/km)
                     --grid      : Slowness grid spacing
                     --win       : Window length in seconds
                     --overlap   : Overlap ratio [0-1]
-
+                    
+                Later, you can point with the mouse over the plot higher power hills and do:
+                
+                press "1" and then "e" for FK slowness map
+                press "2" and then "e"  for FK slowness map
+                press "3" and then "e" for MTP.COHERENCE slowness map
+                press "4" and then "e" for MUSIC slowness map
+                
                 Example:
-                    >> beam --method FK --fmin 1.0 --fmax 3.0 --grid 0.025 --win 3 --overlap 0.05
+                    >> beam --fmin 1.0 --fmax 3.0 --grid 0.025 --win 3 --overlap 0.05
             """,
+
             "pm": """
             pm
                 Perform particle motion analysis on all valid 3-component trace groups
@@ -953,7 +1073,8 @@ class PlotCommandPrompt:
                     >> cwt 2 mh 6 0.5 10
                     >> cwt 2 pa 6 0.5 10 -120
         """,
-        "spectrogram": """
+
+            "spectrogram": """
             spectrogram <index> [<win_sec> <overlap_percent>]
             spec <index> [<win_sec> <overlap_percent>]
                 Plot spectrogram of the selected trace using a moving FFT window, using multitaper.
@@ -972,7 +1093,28 @@ class PlotCommandPrompt:
                     >> spectrogram 1
                     >> spec 0 3.0 75
                     >> spec 0 5.0 50 -120
+        """,
+
+            "smap": """
+            
+            Run slowness map using the FK (or other) beamforming method on all traces.
+            smap [--fmin <Hz>] [--fmax <Hz>] [--smax <s/km>] [--grid <step>] [--method FK]
+                
+                Options:
+                
+                    --method    : Beamforming type ('FK', 'MTP.COHERENCE', 'CAPON' or 'MUSIC), DEFAULT FK
+                    --fmin      : Min frequency (Hz)
+                    --fmax      : Max frequency (Hz)
+                    --smax      : Max slowness (s/km)
+                    --grid      : Slowness grid spacing
+                    --nsignals  : number of expected signals arriving in the time window (at the same time). For MUSIC algorythm
+                
+                Example:
+                    >> smap --method CAPON --fmin 1.0 --fmax 3.0 --grid 0.01
+                    >> smap --method MUSIC --fmin 1.0 --fmax 3.0 --grid 0.01  --nsignals 1
+        
         """
+
         }
 
         # Check if specific command is requested
@@ -989,19 +1131,24 @@ class PlotCommandPrompt:
         print(" p                             Return to interactive picking mode")
         print(" n                             Next set of traces / exit prompt")
         print(" b                             Previous set of traces")
+        print(" load_picks --file <file_path> Load picks from nlloc pick file")
         print(" filter <type> <fmin> <fmax>   Filter traces (type: help filter for details)")
         print(" spectrum <index>|all [type]   Plot amplitude spectrum (loglog, xlog, ylog)")
         print(" spec <idx> [win overlap]      Plot multitaper-spectrogram of trace, (help spectrogram)")
         print(" cwt <idx> <wavelet> <param>   Continuous wavelet transform (help cwt)")
-        print(" beam [--fmin --fmax ...]      Beamforming analysis (type: help beam for options)")
+        print(" beam [--fmin --fmax --overlap ....] Beamforming analysis (type: help beam for options)")
+        print(" smap [--method --fmin --fmax ....] Slowness map (type: help smap for options)")
         print(" plot_type <type>              Change plot mode: standard, record, overlay")
         print(" concat                        Merge/concatenate traces")
         print(" shift --phase <name>          Shift by pick (type: help shift for info)")
         print(" cut --phase <name> ...        Trim traces (type: help cut for usage)")
         print(" write --folder_path <path>    Export displayed traces to HDF5")
+        print(" info                          Print header information from displayed traces")
         print(" exit                          Close command line and exit to interactive picking mode")
         print(" help [command]                Show general or detailed help")
 
+
 if __name__ == "__main__":
     import multiprocessing as mp
-    mp.set_start_method("fork", force=True)  # 'fork' is safest for Matplotlib on macOS
\ No newline at end of file
+
+    mp.set_start_method("fork", force=True)  # 'fork' is safest for Matplotlib on macOS
